/*********************************************************************

    Copyright 2016, Michael Steinert

    This file is part of DrawArea (Dijkstra Routing Algorithm Which
    Ascertains REachable Areas).

    DrawArea is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DrawArea is distributed WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DrawArea.  If not, see <http://www.gnu.org/licenses/>.

***********************************************************************/

#include "DrawArea.h"

// construct Node
Node::Node(size_t alt)
{
	altitude=alt;
}

// add a neighboring node and calculate the time to travel to it
// by applying the DistanceFucntion
void Node::AddNeighbour(int shift, float distance)
{
	if ((this+shift)->altitude==INVALID_ALTITUDE)
		return;
	neighbors.push_back(Edge(this,shift,distance));
}

// construct Edge as pointer-shift and temporal distance
Edge::Edge(Node* start, int shift, float spacialDistance)
{
	this->shift=shift;
	this->distance=Distance(start->altitude,(start+shift)->altitude,spacialDistance);
}

// calculate travel time along the edge based on altitude difference and spacial distance
float Edge::Distance(float startAltitude, float endAltitude, float spacialDistance)
{
	float dHeight=int16_t(endAltitude)-int16_t(startAltitude);
	return MIN_TIME*spacialDistance+(dHeight>0.0?POSITIVE_SLOPE_FACTOR:NEGATIVE_SLOPE_FACTOR)*dHeight;
}

// connect each node with its 8 nearest neighbors (if existing)
void CreateGraphEdges(Node* nodes, size_t numCols, size_t numRows, float distance)
{
	float diagDistance=SQRT_2*distance;
	Node* node=nodes;
	for (size_t i=0; i<numRows; i++)
	{
        	for (size_t j=0; j<numCols; j++)
        	{
    			node->neighbors.clear();
			if (node->altitude==INVALID_ALTITUDE)
			{
				node++;
				continue;
			}

			if (i>0)
			{
				node->AddNeighbour(-numCols,distance);		// northern neighbor
				if (j>0)
					node->AddNeighbour(-numCols-1,diagDistance);	// north-western
				if (j+1<numCols)
					node->AddNeighbour(-numCols+1,diagDistance);	// north-eastern
			}

			if (i+1<numRows)
			{
				node->AddNeighbour(numCols,distance);		// southern neighbor
				if (j>0)
					node->AddNeighbour(numCols-1,diagDistance);	// south-western
				if (j+1<numCols)
					node->AddNeighbour(numCols+1,diagDistance);	// south-eastern
			}

			if (j>0)
				node->AddNeighbour(-1,distance);			// western neighbor
			if (j+1<numCols)
				node->AddNeighbour(1,distance);			// eastern neighbor

			node++;
		}
	}
}
