/*********************************************************************

    Copyright 2016, Michael Steinert

    This file is part of DrawArea (Dijkstra Routing Algorithm Which
    Ascertains REachable Areas).

    DrawArea is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DrawArea is distributed WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DrawArea.  If not, see <http://www.gnu.org/licenses/>.

***********************************************************************/

#include "DrawArea.h"

#include <algorithm>

// constructor:
// 	heap list so we can use heap operations
// 	to have the MapPoint with the smallest
// 	estimated distance in front
OpenList::OpenList()
{
    make_heap(v.begin(),v.end());
    it=v.end();
}

// destructor:
// 	delete all WayPoints in the list
OpenList::~OpenList()
{
	while (!v.empty())
	{
		delete v.back().wayPoint;
		v.pop_back();
	}
}

// add MapPoint to list and ensure heap shape
void OpenList::Push(WayPoint* wp)
{
	MapPoint mp=new WayPoint(*wp);
	v.push_back(mp);
	push_heap(v.begin(),v.end());
	it=v.end();
}


// remove MapPoint with the smallest
// estimated distance from list and return it
WayPoint* OpenList::Pop()
{
	if (v.empty())
		return 0;

	WayPoint* wp=v.front().wayPoint;
	pop_heap(v.begin(),v.end());
	v.pop_back();
	return wp;
}

// update heap structure of list
void OpenList::Update()
{
	make_heap(v.begin(),v.end());
}

// get list size
size_t OpenList::Size()
{
	return v.size();
}

// check if list is empty
bool OpenList::Empty()
{
	return v.empty();
}

// check if MapPoint is contained in list
bool OpenList::Contains(WayPoint* wp)
{
	for (it=v.begin(); it!=v.end(); it++)
		if (*it==wp)
			return true;
	return false;
}

// return a pointer to the WayPoint was wanted
// recently, if it was found
// else return NULL-pointer
WayPoint* OpenList::Select()
{
	if (it==v.end())
		return NULL;
	return it->wayPoint;
}
